@@ -1,9 +1,28 @@
 /**********************************************************************
  *
- * BOX3D IO and conversions
+ * PostGIS - Spatial Types for PostgreSQL
+ * http://postgis.net
+ *
+ * PostGIS is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * PostGIS is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with PostGIS.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ **********************************************************************
+ *
+ * ^copyright^
  *
  **********************************************************************/
 
+
 #include "postgres.h"
 #include "fmgr.h"
 #include "utils/elog.h"
@@ -37,6 +56,7 @@ Datum BOX3D_xmax(PG_FUNCTION_ARGS);
 Datum BOX3D_ymax(PG_FUNCTION_ARGS);
 Datum BOX3D_zmax(PG_FUNCTION_ARGS);
 Datum BOX3D_combine(PG_FUNCTION_ARGS);
+Datum BOX3D_combine_BOX3D(PG_FUNCTION_ARGS);
 
 /**
  *  BOX3D_in - takes a string rep of BOX3D and returns internal rep
@@ -180,69 +200,159 @@ Datum BOX3D_to_LWGEOM(PG_FUNCTION_ARGS)
 	GSERIALIZED *result;
 	POINT4D pt;
 
-
 	/**
 	 * Alter BOX3D cast so that a valid geometry is always
 	 * returned depending upon the size of the BOX3D. The
 	 * code makes the following assumptions:
 	 *     - If the BOX3D is a single point then return a
 	 *     POINT geometry
-	 *     - If the BOX3D represents either a horizontal or
-	 *     vertical line, return a LINESTRING geometry
-	 *     - Otherwise return a POLYGON
+	 *     - If the BOX3D represents a line in any of X, Y
+	 *     or Z dimension, return a LINESTRING geometry
+	 *     - If the BOX3D represents a plane in the X, Y,
+	 *     or Z dimension, return a POLYGON geometry
+	 *     - Otherwise return a POLYHEDRALSURFACE geometry
 	 */
 
-	pa = ptarray_construct_empty(0, 0, 5);
+	pa = ptarray_construct_empty(LW_TRUE, LW_FALSE, 5);
 
-	if ( (box->xmin == box->xmax) && (box->ymin == box->ymax) )
+	/* BOX3D is a point */
+	if ( (box->xmin == box->xmax) && (box->ymin == box->ymax) &&
+			(box->zmin == box->zmax) )
 	{
 		LWPOINT *lwpt = lwpoint_construct(SRID_UNKNOWN, NULL, pa);
 
 		pt.x = box->xmin;
 		pt.y = box->ymin;
+		pt.z = box->zmin;
 		ptarray_append_point(pa, &pt, LW_TRUE);
 
 		result = geometry_serialize(lwpoint_as_lwgeom(lwpt));
+		lwpoint_free(lwpt);
 	}
-	else if (box->xmin == box->xmax ||
-	         box->ymin == box->ymax)
+	/* BOX3D is a line */
+	else if (((box->xmin == box->xmax ||
+			   box->ymin == box->ymax) &&
+			   box->zmin == box->zmax) ||
+		     ((box->xmin == box->xmax ||
+			   box->zmin == box->zmax) &&
+			   box->ymin == box->ymax) ||
+		     ((box->ymin == box->ymax ||
+			   box->zmin == box->zmax) &&
+			   box->xmin == box->xmax))
 	{
 		LWLINE *lwline = lwline_construct(SRID_UNKNOWN, NULL, pa);
 
 		pt.x = box->xmin;
 		pt.y = box->ymin;
+		pt.z = box->zmin;
 		ptarray_append_point(pa, &pt, LW_TRUE);
 		pt.x = box->xmax;
 		pt.y = box->ymax;
+		pt.z = box->zmax;
 		ptarray_append_point(pa, &pt, LW_TRUE);
 
 		result = geometry_serialize(lwline_as_lwgeom(lwline));
+		lwline_free(lwline);
 	}
+	/* BOX3D is a polygon in the X plane */
+	else if (box->xmin == box->xmax)
+	{
+		POINT4D points[4];
+		LWPOLY *lwpoly;
+
+		/* Initialize the 4 vertices of the polygon */
+		points[0] = (POINT4D) { box->xmin, box->ymin, box->zmin };
+		points[1] = (POINT4D) { box->xmin, box->ymax, box->zmin };
+		points[2] = (POINT4D) { box->xmin, box->ymax, box->zmax };
+		points[3] = (POINT4D) { box->xmin, box->ymin, box->zmax };
+
+		lwpoly = lwpoly_construct_rectangle(LW_TRUE, LW_FALSE,
+				&points[0], &points[1], &points[2], &points[3]);
+		result = geometry_serialize(lwpoly_as_lwgeom(lwpoly));
+		lwpoly_free(lwpoly);
+	}
+	/* BOX3D is a polygon in the Y plane */
+	else if (box->ymin == box->ymax)
+	{
+		POINT4D points[4];
+		LWPOLY *lwpoly;
+
+		/* Initialize the 4 vertices of the polygon */
+		points[0] = (POINT4D) { box->xmin, box->ymin, box->zmin };
+		points[1] = (POINT4D) { box->xmax, box->ymin, box->zmin };
+		points[2] = (POINT4D) { box->xmax, box->ymin, box->zmax };
+		points[3] = (POINT4D) { box->xmin, box->ymin, box->zmax };
+
+		lwpoly = lwpoly_construct_rectangle(LW_TRUE, LW_FALSE,
+				&points[0], &points[1], &points[2], &points[3]);
+		result = geometry_serialize(lwpoly_as_lwgeom(lwpoly));
+		lwpoly_free(lwpoly);
+	}
+	/* BOX3D is a polygon in the Z plane */
+	else if (box->zmin == box->zmax)
+	{
+		POINT4D points[4];
+		LWPOLY *lwpoly;
+
+		/* Initialize the 4 vertices of the polygon */
+		points[0] = (POINT4D) { box->xmin, box->ymin, box->zmin };
+		points[1] = (POINT4D) { box->xmin, box->ymax, box->zmin };
+		points[2] = (POINT4D) { box->xmax, box->ymax, box->zmin };
+		points[3] = (POINT4D) { box->xmax, box->ymin, box->zmin };
+
+		lwpoly = lwpoly_construct_rectangle(LW_TRUE, LW_FALSE,
+			   	&points[0], &points[1], &points[2], &points[3]);
+		result = geometry_serialize(lwpoly_as_lwgeom(lwpoly));
+		lwpoly_free(lwpoly);
+	}
+	/* BOX3D is a polyhedron */
 	else
 	{
-		LWPOLY *lwpoly = lwpoly_construct(SRID_UNKNOWN, NULL, 1, &pa);
+		POINT4D points[8];
+		static const int ngeoms = 6;
+		LWGEOM **geoms = (LWGEOM **) lwalloc(sizeof(LWGEOM *) * ngeoms);
+		LWGEOM *geom = NULL;
 
-		pt.x = box->xmin;
-		pt.y = box->ymin;
-		ptarray_append_point(pa, &pt, LW_TRUE);
-		pt.x = box->xmin;
-		pt.y = box->ymax;
-		ptarray_append_point(pa, &pt, LW_TRUE);
-		pt.x = box->xmax;
-		pt.y = box->ymax;
-		ptarray_append_point(pa, &pt, LW_TRUE);
-		pt.x = box->xmax;
-		pt.y = box->ymin;
-		ptarray_append_point(pa, &pt, LW_TRUE);
-		pt.x = box->xmin;
-		pt.y = box->ymin;
-		ptarray_append_point(pa, &pt, LW_TRUE);
+		/* Initialize the 8 vertices of the box */
+		points[0] = (POINT4D) { box->xmin, box->ymin, box->zmin };
+		points[1] = (POINT4D) { box->xmin, box->ymax, box->zmin };
+		points[2] = (POINT4D) { box->xmax, box->ymax, box->zmin };
+		points[3] = (POINT4D) { box->xmax, box->ymin, box->zmin };
+		points[4] = (POINT4D) { box->xmin, box->ymin, box->zmax };
+		points[5] = (POINT4D) { box->xmin, box->ymax, box->zmax };
+		points[6] = (POINT4D) { box->xmax, box->ymax, box->zmax };
+		points[7] = (POINT4D) { box->xmax, box->ymin, box->zmax };
 
-		result = geometry_serialize(lwpoly_as_lwgeom(lwpoly));
-		
+		/* add bottom polygon */
+		geoms[0] = lwpoly_as_lwgeom(lwpoly_construct_rectangle(LW_TRUE, LW_FALSE,
+				   	&points[0], &points[1], &points[2], &points[3]));
+		/* add top polygon */
+		geoms[1] = lwpoly_as_lwgeom(lwpoly_construct_rectangle(LW_TRUE, LW_FALSE,
+				   	&points[4], &points[5], &points[6], &points[7]));
+		/* add left polygon */
+		geoms[2] = lwpoly_as_lwgeom(lwpoly_construct_rectangle(LW_TRUE, LW_FALSE,
+				   	&points[0], &points[1], &points[5], &points[4]));
+		/* add right polygon */
+		geoms[3] = lwpoly_as_lwgeom(lwpoly_construct_rectangle(LW_TRUE, LW_FALSE,
+				   	&points[3], &points[2], &points[6], &points[7]));
+		/* add back polygon */
+		geoms[4] = lwpoly_as_lwgeom(lwpoly_construct_rectangle(LW_TRUE, LW_FALSE,
+				   	&points[0], &points[3], &points[7], &points[4]));
+		/* add front polygon */
+		geoms[5] = lwpoly_as_lwgeom(lwpoly_construct_rectangle(LW_TRUE, LW_FALSE,
+				   	&points[1], &points[2], &points[6], &points[5]));
+
+		geom = (LWGEOM *) lwcollection_construct(POLYHEDRALSURFACETYPE,
+				SRID_UNKNOWN, NULL, ngeoms, geoms);
+
+		FLAGS_SET_SOLID(geom->flags, 1);
+
+		result = geometry_serialize(geom);
+		lwcollection_free((LWCOLLECTION *) geom);
 	}
 
 	gserialized_set_srid(result, box->srid);
+
 	PG_RETURN_POINTER(result);
 }
 
@@ -259,15 +369,37 @@ expand_box3d(BOX3D *box, double d)
 	box->zmax += d;
 }
 
+static void
+expand_box3d_xyz(BOX3D *box, double dx, double dy, double dz)
+{
+	box->xmin -= dx;
+	box->xmax += dx;
+	box->ymin -= dy;
+	box->ymax += dy;
+	box->zmin -= dz;
+	box->zmax += dz;
+}
+
 PG_FUNCTION_INFO_V1(BOX3D_expand);
 Datum BOX3D_expand(PG_FUNCTION_ARGS)
 {
 	BOX3D *box = (BOX3D *)PG_GETARG_POINTER(0);
-	double d = PG_GETARG_FLOAT8(1);
 	BOX3D *result = (BOX3D *)palloc(sizeof(BOX3D));
-
 	memcpy(result, box, sizeof(BOX3D));
-	expand_box3d(result, d);
+
+	if (PG_NARGS() == 2) {
+		/* Expand the box the same amount in all directions */
+		double d = PG_GETARG_FLOAT8(1);
+		expand_box3d(result, d);
+	}
+	else
+	{
+		double dx = PG_GETARG_FLOAT8(1);
+		double dy = PG_GETARG_FLOAT8(2);
+		double dz = PG_GETARG_FLOAT8(3);
+
+		expand_box3d_xyz(result, dx, dy, dz);
+	}
 
 	PG_RETURN_POINTER(result);
 }
@@ -341,7 +473,7 @@ Datum BOX3D_zmax(PG_FUNCTION_ARGS)
 }
 
 /**
-* Used in the ST_Extent and ST_Extent3D aggregates, does not read the 
+* Used in the ST_Extent and ST_Extent3D aggregates, does not read the
 * serialized cached bounding box (since that is floating point)
 * but calculates the box in full from the underlying geometry.
 */
@@ -409,6 +541,34 @@ Datum BOX3D_combine(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(result);
 }
 
+PG_FUNCTION_INFO_V1(BOX3D_combine_BOX3D);
+Datum BOX3D_combine_BOX3D(PG_FUNCTION_ARGS)
+{
+	BOX3D *box0 = (BOX3D*)(PG_ARGISNULL(0) ? NULL : PG_GETARG_POINTER(0));
+	BOX3D *box1 = (BOX3D*)(PG_ARGISNULL(1) ? NULL : PG_GETARG_POINTER(1));
+	BOX3D *result;
+
+	if (box0 && !box1)
+		PG_RETURN_POINTER(box0);
+
+	if (box1 && !box0)
+		PG_RETURN_POINTER(box1);
+
+	if (!box1 && !box0)
+		PG_RETURN_NULL();
+	
+	result = palloc(sizeof(BOX3D));
+	result->xmax = Max(box0->xmax, box1->xmax);
+	result->ymax = Max(box0->ymax, box1->ymax);
+	result->zmax = Max(box0->zmax, box1->zmax);
+	result->xmin = Min(box0->xmin, box1->xmin);
+	result->ymin = Min(box0->ymin, box1->ymin);
+	result->zmin = Min(box0->zmin, box1->zmin);
+	result->srid = box0->srid;
+	
+	PG_RETURN_POINTER(result);
+}
+
 PG_FUNCTION_INFO_V1(BOX3D_construct);
 Datum BOX3D_construct(PG_FUNCTION_ARGS)
 {
